# -*- coding: utf8 -*-

from django.conf import settings
from django.contrib.auth.models import Permission

from accounts.models import User, Group
from ldapaccounts.models import LdapGroup, LdapClub, LdapPosixGroup, LdapUser
from django.contrib import messages


def liste_bureau_user(user):
    if not user.login_inp_net:
        return []

    bureaux = []
    for club in LdapClub.objects.all():
        if user.login_inp_net in club.membres_bureau():
            bureaux.append(club.cn)
    return bureaux


def liste_groupe_user(user):
    if not user.login_inp_net:
        return []

    groupes = []
    for groupe in LdapGroup.objects.all():
        if user.login_inp_net in groupe.membres:
            groupes.append((groupe.cn, groupe.displayName))
    return groupes


def update_user(user, update_mail=False):

    if not user.login_inp_net:
        return

    user_ldap = user.get_LdapUser()
    user.ecole = user_ldap.get_ecole().o
    if update_mail:
        if not User.objects.filter(email=user_ldap.mail).exists():
            user.email = user_ldap.mail
        else:
            print(
                "Impossible de modifier l'adresse de {} ({}). Un compte avec {} existe déjà.".format(
                    user.login_inp_net, user.email, user_ldap.mail
                )
            )

    user.first_name = user_ldap.prenom
    user.last_name = user_ldap.nom
    user.is_adherent_ae = user_ldap.inscrit_AE()
    user.promo = user_ldap.promo
    user.is_active = True

    membre_bureau_clubs = liste_bureau_user(user)
    p = Permission.objects.get(codename="add_event")
    if membre_bureau_clubs:
        user.user_permissions.add(p)
    else:
        user.user_permissions.remove(p)
    groups_user = [g.cn for g in user.groupes.all()]
    groups_user_ldap = liste_groupe_user(user)
    update_groups_user_from_dict(user, groups_user, groups_user_ldap)
    managed_clubs_user = [c.cn for c in user.managed_clubs.all()]
    update_managed_clubs_user_from_dict(user, managed_clubs_user, membre_bureau_clubs)
    if settings.IMPORT_SUPERUSER and not settings.IMPORT_SUPERUSER_CAS:
        user.is_superuser = user.login_inp_net in LdapPosixGroup.objects.get(cn="admin-inp").membres
    user.set_unusable_password()
    user.save()


def update_groups_user_from_dict(user, groups_user, groups_user_ldap):
    for g in groups_user_ldap:
        try:
            g_db = Group.objects.get(cn=g[0])
        except Group.DoesNotExist:
            g_db = Group(cn=g[0], displayName=g[1])
            g_db.save()
        if g_db.cn not in groups_user:
            user.groupes.add(g_db)
    for g in groups_user:
        if g not in [g_ldap[0] for g_ldap in groups_user_ldap]:
            g_db = Group.objects.get(cn=g)
            user.groupes.remove(g_db)
    user.save()


def update_managed_clubs_user_from_dict(user, managed_clubs_user, managed_clubs_user_ldap):
    for c in managed_clubs_user_ldap:
        if c not in managed_clubs_user:
            club = Group.objects.get(cn=c)
            user.managed_clubs.add(club)
    for c in managed_clubs_user:
        if c not in managed_clubs_user_ldap:
            club = Group.objects.get(cn=c)
            user.managed_clubs.remove(club)
            pass
    user.save()


def conflit_associate(user, user_ldap):
    if User.objects.filter(login_inp_net=user_ldap.login_inp_net).exists():
        return "Conflit de login INP-net"

    if User.objects.filter(email=user_ldap.mail).exists():
        return "Conflit d'adresse mail"

    return None


def get_ldap_user(login_inp_net):
    if not login_inp_net:
        return (None, (lambda request: None))
    user = User.objects.filter(login_inp_net=login_inp_net)
    if user.exists() and len(user) == 1:
        return (user.first(), lambda request: None)

    ldap_user = LdapUser.objects.filter(login_inp_net=login_inp_net)
    if ldap_user.exists():
        ldap_user = ldap_user.first()
        if not User.objects.filter(email=ldap_user.mail).exists():
            user = User(
                email=ldap_user.mail,
                login_inp_net=ldap_user.login_inp_net,
                first_name=ldap_user.prenom,
                last_name=ldap_user.nom,
            )
            user.save()
            update_user(user)
            user.save()
            if user:
                return (user, (lambda request: messages.info(request, "Compte INP-net importé depuis le ldap.")))
        return (
            None,
            (
                lambda request: messages.error(
                    request, "Un problème survient lors de l'import de ce login INP-net. Contacter INP-net."
                )
            ),
        )
    else:
        return (None, (lambda request: messages.error(request, "Ce login INP-net n’a pas été trouvé.")))
